"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeNewLibrary = void 0;
const fs_1 = require("fs");
const string_1 = require("../augments/string");
const library_migration_error_1 = require("../errors/library-migration-error");
const library_parse_error_1 = require("../errors/library-parse-error");
function makeNewLibrary({ oldLibrary, replacePaths, checkReplacementPaths = true, loggingEnabled = true, checkFiles = false, removeRatingComputed = false, }) {
    const unreplacedPaths = new Set();
    const replacePathUsage = replacePaths.map(() => 0);
    const missingFiles = [];
    loggingEnabled && console.info(`Replacing locations...`);
    const newLibrary = {
        ...oldLibrary,
        Tracks: Object.keys(oldLibrary.Tracks).reduce((newTracks, trackKey) => {
            const oldTrack = oldLibrary.Tracks[trackKey];
            if (!oldTrack) {
                throw new library_parse_error_1.LibraryParseError(`Track key "${trackKey}" didn't actually exist in the library`);
            }
            let rawNewTrack = {
                ...oldTrack,
            };
            if (removeRatingComputed) {
                delete rawNewTrack['Rating Computed'];
            }
            const originalLocation = rawNewTrack.Location;
            let markedForDeletion = false;
            if (originalLocation) {
                const updatingTrack = {
                    ...rawNewTrack,
                };
                const replaced = replacePaths.reduce((replaced, replacePath, replacePathIndex) => {
                    let used = false;
                    if (updatingTrack.Location.includes(replacePath.old) ||
                        string_1.decodeLocation(updatingTrack.Location).includes(replacePath.old)) {
                        if ('delete' in replacePath && replacePath.delete) {
                            markedForDeletion = true;
                            used = true;
                        }
                        else if ('new' in replacePath && replacePath.new) {
                            updatingTrack.Location = updatingTrack.Location.replace(replacePath.old, replacePath.new);
                            if (updatingTrack.Location === updatingTrack.Location &&
                                !updatingTrack.Location.startsWith('http')) {
                                // try with sanitization
                                updatingTrack.Location = string_1.encodeLocation(string_1.decodeLocation(updatingTrack.Location).replace(replacePath.old, replacePath.new));
                            }
                            used = true;
                        }
                        else {
                            throw new library_migration_error_1.LibraryMigrationError(`Replace path contained neither valid .new or valid .delete: ${JSON.stringify(replacePath)}`);
                        }
                    }
                    if (used) {
                        ++replacePathUsage[replacePathIndex];
                    }
                    return replaced || used;
                }, false);
                if (replaced) {
                    if (!markedForDeletion &&
                        checkFiles &&
                        !updatingTrack.Location.startsWith('http') &&
                        !fs_1.existsSync(string_1.decodeLocation(updatingTrack.Location))) {
                        missingFiles.push({
                            old: string_1.decodeLocation(originalLocation),
                            new: string_1.decodeLocation(updatingTrack.Location),
                        });
                    }
                }
                else {
                    unreplacedPaths.add(originalLocation);
                }
                rawNewTrack = updatingTrack;
            }
            if (!markedForDeletion) {
                newTracks[trackKey] = rawNewTrack;
            }
            return newTracks;
        }, {}),
    };
    loggingEnabled && console.info(`Replacing finished`);
    const errors = [];
    if (checkReplacementPaths) {
        unreplacedPaths.forEach((path) => {
            errors.push(`This track location was not replaced:\n\t\t\t${path}`);
        });
        replacePathUsage.forEach((usage, index) => {
            if (!usage) {
                errors.push(`The following replacement was never used:\n\t\t\t${JSON.stringify(replacePaths[index], null, 4)}`);
            }
        });
    }
    if (missingFiles.length) {
        missingFiles.forEach((missingFile) => {
            errors.push(`\tMissing file:\n\t\t${JSON.stringify(missingFile, null, '\t\t\t').replace(/}$/, '\t\t}')}`);
        });
    }
    if (errors.length) {
        throw new library_migration_error_1.LibraryMigrationError('\n' + errors.join('\n'));
    }
    return newLibrary;
}
exports.makeNewLibrary = makeNewLibrary;
