"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertValidLibrary = void 0;
const library_validation_error_1 = require("../../errors/library-validation-error");
const validation_types_1 = require("./validation-types");
function validateObject(name, input, validationBasis, nameTree) {
    const errors = [];
    if (input === undefined) {
        errors.push(new library_validation_error_1.LibraryValidationError(`${name} does not exist.`, nameTree.concat(name)));
    }
    else if (typeof input !== 'object' || !input) {
        errors.push(new library_validation_error_1.LibraryValidationError(`${name} is not an object: ${input}`, nameTree.concat(name)));
    }
    else {
        const invalidKeys = Object.keys(input).filter((libraryKey) => {
            var _a;
            const validationComparison = validationBasis.types[libraryKey];
            if (!validationComparison) {
                // if we don't have the key in validation checks, that will be caught later
                return false;
            }
            const value = input[libraryKey];
            if (validationComparison.type === 'Date') {
                if (!(value instanceof Date)) {
                    return true;
                }
            }
            else if (validationComparison.type === 'Buffer') {
                if (!(value instanceof Buffer)) {
                    return true;
                }
            }
            else if (typeof value !== validationComparison.type) {
                return true;
            }
            if (validationComparison.type === 'object') {
                const subValidation = (_a = validationBasis.subTypes) === null || _a === void 0 ? void 0 : _a[libraryKey];
                if (subValidation) {
                    Object.keys(value).forEach((subKey) => {
                        const subValue = value[subKey];
                        errors.push(...validateObject(libraryKey, subValue, subValidation, nameTree.concat(name)));
                    });
                }
                else {
                    errors.push(new library_validation_error_1.LibraryValidationError(`Missing validation basis for ${libraryKey} key in ${name}`, nameTree.concat(name)));
                }
            }
            return false;
        });
        const missingValidationKeys = Object.keys(input).filter((libraryKey) => {
            return !(libraryKey in validationBasis.types);
        });
        const missingKeys = Object.keys(validationBasis.types).filter((validationKey) => {
            return !(validationKey in input) && validationBasis.types[validationKey].required;
        });
        const invalidValues = invalidKeys.reduce((accum, key) => {
            var _a;
            accum[key] = {
                expected: ((_a = validationBasis.types[key]) === null || _a === void 0 ? void 0 : _a.type) || 'undefined',
                got: typeof input[key],
            };
            return accum;
        }, {});
        const invalidKeysMessage = invalidKeys.length
            ? `Invalid ${name} keys:\n\t\t${JSON.stringify(invalidValues, null, 4)
                .split('\n')
                .join('\n\t\t')}`
            : '';
        const missingKeysMessage = missingKeys.length
            ? `Missing ${name} keys:\n\t\t${missingKeys.join(', ')}`
            : '';
        const missingValidationKeysMessage = missingValidationKeys.length
            ? `Missing ${name} validation for keys:\n\t\t${missingValidationKeys.join(', ')}`
            : '';
        const errorMessages = [
            invalidKeysMessage,
            missingKeysMessage,
            missingValidationKeysMessage,
        ].filter((message) => !!message);
        if (errorMessages.length) {
            errors.push(new library_validation_error_1.LibraryValidationError(`${input.Name} errors:\n\t${errorMessages.join('\n\t')}`, nameTree.concat(name)));
        }
    }
    return errors;
}
function assertValidLibrary(parsedLibrary) {
    const validationErrors = validateObject('whole library', parsedLibrary, {
        types: validation_types_1.parsedLibraryTypes,
        subTypes: {
            Tracks: { types: validation_types_1.parsedTrackTypes },
            Playlists: {
                types: validation_types_1.parsedPlaylistTypes,
                subTypes: {
                    'Playlist Items': { types: validation_types_1.parsedPlaylistItemTypes },
                },
            },
        },
    }, []);
    if (validationErrors.length) {
        throw new library_validation_error_1.LibraryValidationError('\n' + validationErrors.map((validationError) => validationError.message).join('\n'));
    }
}
exports.assertValidLibrary = assertValidLibrary;
